-------------------------------------------------------------------------------
--
-- Title       : fibonacciero3
-- Design      : fibonacciero3
-- Author      : Ney Calazans
-- Company     : PUCRS - Faculty of Informatics
--
-------------------------------------------------------------------------------
--
-- File        : fibonacciero3.vhd
-- Generated   : Fri Sep 26 09:21:50 2008
-- From        : interface description file
-- By          : Itf2Vhdl ver. 1.20
--
-------------------------------------------------------------------------------
--
-- Description:	Hardware que computa os nro_elementos
-- 				primeiros elementos da Srie de Fibonacci 
--				O processo de gerao pode ser temporariamente
--				interrompido a qualquer instante.
--
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_unsigned.all;

entity fibonacciero3 is
	 port(
		 clock : in STD_LOGIC;
		 reset : in STD_LOGIC;		
		 anda_para : in STD_LOGIC;	
		 nro_elementos : in STD_LOGIC_VECTOR(7 downto 0);
		 saida : out STD_LOGIC_VECTOR(7 downto 0)		 
	     );
end fibonacciero3;

--}} End of automatically maintained section

architecture fibonacciero3 of fibonacciero3 is
signal soma, soma_old, saida_int, cont : STD_LOGIC_VECTOR(7 downto 0);
signal ce, zero : STD_LOGIC;
type states is (Start, C_P, Fim);
signal EA, PE : states;
begin
	
	saida <= saida_int; 
	soma <= saida_int + soma_old; 	
	
--------------------------------------------------	
-- Bloco de Dados
--------------------------------------------------	
-- Um somador de duas entradas e dois registradores,
-- formando um pipeline de aramazenamento e clculo de
-- 3 elementos consecutivos da srie.
Bl_dados: process (clock, reset)
	begin			
		if reset='1' then
			soma_old <= (0=>'1', others=>'0');
			saida_int <= (others=>'0');
		elsif clock'event and clock='1' then 
			if ce='1' then
				soma_old <= soma;
				saida_int <= soma_old;
			end if;
		end if;
	end process;
	
--------------------------------------------------	
-- Bloco de Controle
--------------------------------------------------	
-- Mquina de estados com dois processos: um para
-- o registrador de estados e um para as funes de
-- transio de estados e de clculo da sada
Reg_estado: process (clock, reset) 	
	begin	   
	   if reset='1' then
		   EA <= Start;
	   elsif clock'event and clock='1' then
		   EA <= PE;
	   end if;		   
	end process; 

Func_S_PE: process (EA, zero, anda_para)
	begin
		case EA is 
         when Start =>  ce <='0'; 
			 if zero='0' then
			 	PE <= C_P; 
			 else PE <= Fim; 
			 end if; 
         when C_P =>  
			 if zero='0' then
			 	PE <= C_P; 
			 else PE <= Fim; 
			 end if; 
			 if anda_para='1' and cont/=x"00" then	
			 	ce<='1';
			 else ce<='0';
			 end if;	
		when Fim =>  	
			ce <='0';
			PE <= Fim;
		end case; 
	end process;

-- Contador auxiliar com deteco de zero combinacional
-- inicializado com nro_elementos-1 exceto se valor inicial
-- for 0, em cujo caso incializa-se com 0 mesmo.
contador: process (clock, reset)
	begin			
		if reset='1' then
			if nro_elementos /=x"00" then
					cont <= nro_elementos-1;
			else 	cont <= x"00";
			end if;
		elsif clock'event and clock='1' then 
			if ce='1' then
				cont <= cont-1;
			end if;
		end if;
	end process;   
	
	zero <= '1' when cont=x"00" else '0'; 
		
end fibonacciero3;
